@@ -2,21 +2,25 @@
 
 import gtk,os,getpass,sys
 from xml.dom import minidom, Node
+exitx = False
 
 class inputconf:
 	if os.name == "nt":
 		CFG_FILENAME = "./input.cfg"
 		MOUSE_SETTING_FILE = "C:\\python24\\pdaxrom\\misc\\x11"
 		MODMAP_FILE = "C:\\python24\\pdaxrom\\misc\\userdefined.xmodmap"
+		KERNELMAP_FILE = "C:\\python24\\pdaxrom\\misc\\kernel.map"
 		RC_XML = "C:\\python24\\pdaxrom\\misc\\rc.xml"
 	else:
 		CFG_FILENAME = os.environ['HOME'] + "/Choices/input.cfg"
 		MOUSE_SETTING_FILE = "/etc/sysconfig/x11"
 		MODMAP_FILE = "/etc/X11/kb/userdefined.xmodmap"
+		KERNELMAP_FILE = "/etc/sysconfig/keyboard/kernel.map"
 		RC_XML = "/etc/xdg/openbox/rc.xml"
 
 	def __init__(self):
-		if (getpass.getuser().lower() != "root"):
+
+		if (getpass.getuser().lower() != "root" and os.name != "nt"):
 			Warningdialog = gtk.MessageDialog(None,gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,gtk.MESSAGE_WARNING,gtk.BUTTONS_OK,"You need to be root in order to run this program.")
 			Warningdialog.run()
 			Warningdialog.destroy()
@@ -89,14 +93,14 @@ class inputconf:
 		# altbox and ctrlbox are mutually exclusive because they use the same ListStore,
 		# but not mutually exclusive with regards to rmbox and mmbox as they use another ListStore!
 		i = 0
-		for strtxt in self.store:
+		for strtxt in self.store1:
 			if (strtxt[1] == self.alt_mod):
 				break
 			i += 1
 		altbox,altcombo = self.build_combobox(self.store1,self.changed,"Alt key is mapped to : ",i,"ALT")
 		
 		i = 0
-		for strtxt in self.store:
+		for strtxt in self.store1:
 			if (strtxt[1] == self.ctrl_mod):
 				break
 			i += 1
@@ -121,6 +125,12 @@ class inputconf:
 		vbox.pack_start(self.rmbox, gtk.FALSE, gtk.FALSE)
 		vbox.pack_start(mmbox, gtk.FALSE, gtk.FALSE)
 
+		hbox = gtk.HBox(gtk.FALSE)
+		vbox.pack_start(hbox, gtk.FALSE, gtk.TRUE)
+		self.clearcalib_but = gtk.CheckButton("Recalibrate stylus on next start of window manager.") 
+		self.clearcalib_but.set_active(gtk.FALSE)
+		hbox.pack_start(self.clearcalib_but, gtk.FALSE, gtk.FALSE)
+
 		vbox2 = gtk.VBox(homogeneous=gtk.TRUE, spacing=10)
 		vbox2.set_border_width(10)
 		label = gtk.Label("Here you can configure which keys the control(ctrl)\nand alt buttons are mapped onto.")
@@ -195,8 +205,8 @@ class inputconf:
 		vbox5.pack_start(label, gtk.FALSE, gtk.TRUE)
 
 		self.scrollwheeldirbox = gtk.combo_box_new_text()
-	      	self.scrollwheeldirbox.append_text("Scroll up and down")
-	      	self.scrollwheeldirbox.append_text("Scroll left and right")
+	      	self.scrollwheeldirbox.append_text("Arrow key up and down")
+	      	self.scrollwheeldirbox.append_text("Arrow key left and right")
 	      	self.scrollwheeldirbox.set_size_request(250,30)
 	      	if (self.scrollwheeldir == "updown"):
 	      		self.scrollwheeldirbox.set_active(0)
@@ -401,7 +411,7 @@ class inputconf:
 
 		model = self.scrollwheeldirbox.get_model()
 		active = self.scrollwheeldirbox.get_active()
-		if (model[active][0].lower() == "scroll up and down"):
+		if (model[active][0].lower() == "arrow key up and down"):
 			input_node.setAttribute("SCROLLWHEELDIR", "updown")
 			self.scrollwheeldir = "updown"
 		else:
@@ -459,9 +469,11 @@ class inputconf:
 		FD = open(self.CFG_FILENAME, "w")
 		FD.write(conf_tree.toprettyxml())
 		FD.close()
+		conf_tree.unlink()
 		return True
 
 	def close_and_update(self,widget):
+		global exitx
 		self.lock_others()
 		self.save_prefs()
 	
@@ -585,29 +597,54 @@ class inputconf:
 			FD = open(self.RC_XML, "w")
 			FD.write(xmldoc.toprettyxml("",""))
 			FD.close()
+			xmldoc.unlink()
 
 	# Create xmodmap file for keys
-		output += "\n\nclear Control\n"
-		output += "remove Control = Control_L\n"
-		output += "keycode " + str(bcb.store1.get_data("ALT")) + " = Alt_L \n"
-		output += "keycode " + str(bcb.store1.get_data("CTRL")) + " = Control_L \n\n"
-		output += "add Control = Control_L\n\n"
 		f=open(self.MODMAP_FILE, "w")
+		
+		output += "keycode " + str(bcb.store1.get_data("CTRL")) + " = Control_L\n"	
+		output += "keycode " + str(bcb.store1.get_data("ALT")) + " = Alt_L"
+		f.write(output)
+		f.close()
+
+	# Create kernel keymap file for keys
+		output = """keycode 65 = Tab\n\talt\tkeycode 65 = Caps_Lock\n\tshift\tkeycode 65 = Caps_Lock\n\tcontrol\tkeycode 65 = Caps_Lock\n"""
+		for strtxt in self.store1:
+			if ((strtxt[1]-8) == int(bcb.store1.get_data("ALT"))-8):
+				output += "keycode " + str(int(bcb.store1.get_data("ALT"))-8) + " = Alt \n"
+			elif ((strtxt[1]-8) == int(bcb.store1.get_data("CTRL"))-8):
+				output += "keycode " + str(int(bcb.store1.get_data("CTRL"))-8) + " = Control \n"
+			else:
+				if ((strtxt[1]-8) == 27):
+					output += "keycode 27 = Shift\n"
+				elif ((strtxt[1]-8) == 30):
+					output += "keycode 30 = Alt\n"
+				elif ((strtxt[1]-8) == 103):
+					output += "keycode 103 = Shift\n"
+				else:
+					output += "keycode " + str(strtxt[1]-8) + " = nul\n"
+		f=open(self.KERNELMAP_FILE, "w")
 		f.write(output)
 		f.close()
 		
-		os.system("xmodmap " + self.MODMAP_FILE)
+		if (self.clearcalib_but.get_active()):
+			os.system("rm /etc/pointercal")
 		
+		os.system("xmodmap " + self.MODMAP_FILE)
 		Warningdialog = gtk.MessageDialog(self.window,gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,gtk.MESSAGE_WARNING,gtk.BUTTONS_OK,"You have to exit and start the window manager again (with startx) in order to get new mouse key mappings to work.")
 		Warningdialog.run()
 		Warningdialog.destroy()
 		
+		exitx = True
+		self.window.destroy()
 		gtk.main_quit()
 
-def main():	
+def main():
 	gtk.main()
 	return
 
 if __name__	== "__main__":
 	bcb	= inputconf()	
-	main()
\ No newline at end of file
+	main()
+	if (exitx):
+		os.system("eXit")
\ No newline at end of file
